@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# 2011-2012 Nico Schottelius (nico-cdist at schottelius.org)
+# 2011-2013 Nico Schottelius (nico-cdist at schottelius.org)
 # 2012 Steven Armstrong (steven-cdist at armstrong.cc)
 #
 # This file is part of cdist.
@@ -28,20 +28,33 @@ import sys
 import cdist
 from cdist import core
 
+class MissingRequiredEnvironmentVariableError(cdist.Error):
+    def __init__(self, name):
+        self.name = name
+        self.message = "Emulator requires the environment variable %s to be setup" % self.name
+
+    def __str__(self):
+        return self.message
+
+
 class Emulator(object):
     def __init__(self, argv, stdin=sys.stdin.buffer, env=os.environ):
         self.argv           = argv
         self.stdin          = stdin
         self.env            = env
 
-        self.object_id      = False
+        self.object_id      = ''
 
-        self.global_path    = self.env['__global']
-        self.target_host    = self.env['__target_host']
+        try:
+            self.global_path    = self.env['__global']
+            self.target_host    = self.env['__target_host']
 
-        # Internally only
-        self.object_source  = self.env['__cdist_manifest']
-        self.type_base_path = self.env['__cdist_type_base_path']
+            # Internally only
+            self.object_source  = self.env['__cdist_manifest']
+            self.type_base_path = self.env['__cdist_type_base_path']
+
+        except KeyError as e:
+            raise MissingRequiredEnvironmentVariableError(e.args[0])
 
         self.object_base_path = os.path.join(self.global_path, "object")
 
@@ -54,10 +67,10 @@ class Emulator(object):
         """Add hostname and object to logs via logging Filter"""
 
         prefix = self.target_host + ": (emulator)"
-
-        if self.object_id:
-            prefix = prefix + " " + self.type_name + "/" + self.object_id
-
+        prefix = '{0}: emulator {1}'.format(
+            self.target_host,
+            core.CdistObject.join_name(self.type_name, self.object_id)
+        )
         record.msg = prefix + ": " + record.msg
 
         return True
@@ -122,9 +135,7 @@ class Emulator(object):
 
     def setup_object(self):
         # Setup object_id - FIXME: unset / do not setup anymore!
-        if self.cdist_type.is_singleton:
-            self.object_id = "singleton"
-        else:
+        if not self.cdist_type.is_singleton:
             self.object_id = self.args.object_id[0]
             del self.args.object_id
 
@@ -184,7 +195,12 @@ class Emulator(object):
                 if len(requirement) == 0: continue
 
                 # Raises an error, if object cannot be created
-                cdist_object = self.cdist_object.object_from_name(requirement)
+                try:
+                    cdist_object = self.cdist_object.object_from_name(requirement)
+                except core.cdist_type.NoSuchTypeError:
+                    self.log.error("%s requires object %s with non-existing type at %s"  % (self.cdist_object.name, requirement, self.object_source))
+                    raise
+
 
                 self.log.debug("Recording requirement: " + requirement)
 
